-- YGREC8/ALU8.vhdl
-- created mer. nov.  8 07:21:19 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version ven. nov. 10 06:44:34 CET 2017
-- version jeu. nov. 16 02:21:35 CET 2017 : uses SLV shorthands from YGREC8/YGREC8_def.vhdl
-- version mar. déc.  5 21:48:39 CET 2017 : removed F3 input
-- Released under the GNU AGPLv3 license
--
-- This is the arithmetic and logic unit of YGREC8,
-- performing the add/sub and ROP2 operations on 8 bits.
-- This version has separate outputs for ROP2 and ADD 
--
-- OPCODES:
-- F3 F2 F1 F0      tested
-- 0  0  0  0  OR     *
-- 0  0  0  1  XOR    *
-- 0  0  1  0  AND    *
-- 0  0  1  1  ANDN   *
-- 0  1  0  0  CMPU
-- 0  1  0  1  CMPS
-- 0  1  1  0  SUB    *
-- 0  1  1  1  ADD    *

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;

entity ALU8 is
  port( F0, F1, F2, Cin : in SL;
    SRC, DST : in SLV8;
    ROP2, SUM : out SLV8;
    Cout : out SL);
end ALU8;

architecture CLA3 of ALU8 is
  Signal  ALU_XOR, P, G, S, C,
    XOR_SRC, XOR_DST,  -- syntax shortcut for VHDL...
    SRCX, DSTX : SLV8;
  Signal P2, G2 : SLV2;
  Signal C2, compare_signed, negate_DST : SL;

begin
  -- Prepare the control signals:
  compare_signed <=F0 and not F1 and F2; -- CMPS="0101"
  negate_DST <= (F0 AND F1) XOR F2;

  -- Initial XOR of the operands
  XOR_DST <= (7=> negate_DST and not compare_signed, others => negate_DST);
  XOR_SRC <= (7=>                    compare_signed, others =>        '0');
  DSTX <= DST XOR XOR_DST;
  SRCX <= SRC XOR XOR_SRC;

  -- ROP2
  ALU_XOR <= SRCX XOR DSTX;
  G       <= SRCX AND DSTX;
  P       <= SRCX  OR DSTX;
  ROP2    <= G when F1='1'
        else P when F0='0'
        else ALU_XOR;

  -- ADD/SUB, Carry-Lookahead adder with 2-levels of CLA3 blocks
  C(0) <= Cin;

  --inputs: Cin, G0, G1, G2, P0, P1, P2
  C(1)  <= G(0)  OR (Cin AND P(0));
  C(2)  <= G(1)  OR (Cin AND P(0) AND P(1))           OR (G(0) AND P(1));
  P2(0) <=                   P(0) AND P(1) AND P(2);
  G2(0) <= G(2)                                       OR (G(0) AND P(1) and P(2))  OR (G(1) AND P(2));

  C(3)  <= G2(0) OR (Cin AND P2(0));

  --inputs: C3, G3, G4, G5, P3, P4, P5
  C(4)  <= G(3)  OR (C(3) AND P(3));
  C(5)  <= G(4)  OR (C(3) AND P(3) AND P(4))           OR (G(3) AND P(4));
  P2(1) <=                    P(3) AND P(4) AND P(5);
  G2(1) <= G(5)                                        OR (G(3) AND P(4) and P(5))  OR (G(4) AND P(5));

  C(6)  <= G2(1) OR (C(3) AND P2(1));

  -- 2nd level of CLA
  C2    <= G2(1) OR (Cin AND P2(0) AND P2(1)) OR (G2(0) AND P2(1));

  --inputs: C2, G6, G7, P6, P7
  C(7) <= G(6) OR (C2 AND P(6));
  Cout <= G(7) OR (C2 AND P(6) AND P(7)) OR (G(6) AND P(7));
  SUM <= ALU_XOR XOR C;

end CLA3;
