-- YGREC8/ALU8.vhdl
-- created mer. nov.  8 07:21:19 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version ven. nov. 10 07:36:19 CET 2017 : built-in + operator used.
-- version jeu. nov. 16 02:21:35 CET 2017 : uses SLV shorthands from YGREC8/YGREC8_def.vhdl
-- version mar. déc.  5 21:48:39 CET 2017 : removed F3 input
-- Released under the GNU AGPLv3 license
--
-- This is the arithmetic and logic unit of YGREC8,
-- performing the add/sub and boolean operations on 8 bits.
-- This version has separate outputs for ROP2 and ADD
--
-- OPCODES:
-- F3 F2 F1 F0      tested
-- 0  0  0  0  OR     *
-- 0  0  0  1  XOR    *
-- 0  0  1  0  AND    *
-- 0  0  1  1  ANDN   *
-- 0  1  0  0  CMPU
-- 0  1  0  1  CMPS
-- 0  1  1  0  SUB    *
-- 0  1  1  1  ADD    *

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;

entity ALU8 is
  port( F0, F1, F2, Cin : in SL;
    SRC, DST : in SLV8;
    ROP2, SUM : out SLV8;
    Cout : out SL);
end ALU8;

architecture classic of ALU8 is
  Signal  ALU_XOR, P, G,
    XOR_SRC, XOR_DST,
    SRCX, DSTX : SLV8;
  Signal compare_signed, negate_DST : SL;
  Signal sumAux : unsigned(9 downto 0);

begin
  -- Prepare the control signals:
  compare_signed <=F0 and not F1 and F2; -- CMPS="0101"
  negate_DST <= (F0 AND F1) XOR F2;

  -- Initial XOR of the operands
  XOR_DST <= (7=> negate_DST and not compare_signed, others => negate_DST);
  XOR_SRC <= (7=>                    compare_signed, others =>        '0');
  DSTX <= DST XOR XOR_DST;
  SRCX <= SRC XOR XOR_SRC;

  -- ROP2
  ALU_XOR <= SRCX XOR DSTX;
  G       <= SRCX AND DSTX;
  P       <= SRCX  OR DSTX;
  ROP2    <= G when F1='1'
        else P when F0='0'
        else ALU_XOR;

  -- ADD/SUB, the classic way:
  sumAux <= unsigned('0' & SRCX & '1') + unsigned('0' & DSTX & Cin);
  SUM <= std_logic_vector(sumAux(8 downto 1));
  Cout <= std_logic(sumAux(9));

end classic;
