-- YGREC8/ALU8_tb.vhdl
-- version lun. nov. 13 01:43:33 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version jeu. nov. 16 02:21:35 CET 2017 : uses SLV shorthands from YGREC8/YGREC8_def.vhdl
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;
    use work.ygrec8_utils.all;

entity ALU8_tb is
end ALU8_tb;

architecture plip of ALU8_tb is
  signal Cin, Cout : SL := '0';
  signal F : SLV4 := "0000";
  signal SRC, DST, ROP2, SUM : SLV8 := "00000000";
begin

  tb: entity ALU8 port map (
     F0 => F(0),
     F1 => F(1),
     F2 => F(2),
     Cin => Cin,
     SRC => SRC,
     DST => DST,
     SUM => SUM,
     ROP2 => ROP2,
     Cout => Cout);

  bench: process
    variable j, k, errors: integer := 0;
    variable expected : SLV8;
    variable t : SLV10;

    procedure ShowSignals is
    begin
      report " F=" & SLV_to_bin(std_logic_vector(F))
           & " SRC=" & SLV_to_bin(SRC)
           & " DST=" & SLV_to_bin(DST)
           & " Cin=" & std_logic'image(Cin)
           & " Cout=" & std_logic'image(Cout)
           & " SUM=" & SLV_to_bin(SUM)
           & " ROP2=" & SLV_to_bin(ROP2)
           & " expect=" & SLV_to_bin(expected)
           & " Diff=" & SLV_to_bin(t(8 downto 0) XOR (Cout & SUM));
    end ShowSignals;

  begin

    for j in 0 to 255 loop
      SRC <= std_logic_vector(to_unsigned(j, 8));
      for k in 0 to 255 loop
        DST <= std_logic_vector(to_unsigned(k, 8));

        -- Bool:

        F <= Op_OR;
        wait for 1 ns;
        expected := SRC  OR      DST;
        if expected /= ROP2 then ShowSignals; errors:=errors+1; end if;

        F <= Op_XOR;
        wait for 1 ns;
        expected := SRC XOR      DST;
        if expected /= ROP2 then ShowSignals; errors:=errors+1; end if;

        F <= Op_AND;
        wait for 1 ns;
        expected := SRC AND      DST;
        if expected /= ROP2 then ShowSignals; errors:=errors+1; end if;

        F <= Op_ANDN;
        wait for 1 ns;
        expected := SRC AND (NOT DST);
        if expected /= ROP2 then ShowSignals; errors:=errors+1; end if;

        -- ADD:
        F <= Op_ADD;
        Cin <= '0';
        wait for 1 ns;
        t := std_logic_vector(to_unsigned(j+k, 10));
        expected := t(7 downto 0);
        if t /= ('0' & Cout & SUM) then ShowSignals; errors:=errors+1; end if;

        Cin <= '1';
        wait for 1 ns;
        t := std_logic_vector(to_unsigned(j+k+1, 10));
        expected := t(7 downto 0);
        if t /= ('0' & Cout & SUM) then ShowSignals; errors:=errors+1; end if;

        -- SUB:
        F <= Op_SUB;
        --Cin <= '1';
        wait for 1 ns;
        t := std_logic_vector(to_unsigned((j+512)-k, 10));
        expected := t(7 downto 0);
        if t(8 downto 0) /= ( not Cout & SUM) then ShowSignals; errors:=errors+1; end if;

      end loop;
    end loop;

    assert errors = 0
      report integer'image(errors) & " UNEXPECTED RESULTS "
      severity failure;

    wait;
  end process;

end plip;
