-- file YGREC8/ASM_tb.vhdl
-- created ven. nov. 17 00:49:02 CET 2017 by Yann Guidon / ygdes.com
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;
    use work.ygrec8_utils.all;
    use work.ygrec8_asm.all;

entity mustfail is
  generic (
    test_str: string := "0"  -- changed by the -gtest_str=number command line option (see ../t37/ )
  );
end mustfail;

architecture fail of mustfail is
begin

  test: process
    variable test_int : integer := 0;
    variable result : SLV16;
  begin
    test_int := Num_to_int(test_str, "generic parameter/Test number : ");
--    report test_str & " = " & integer'image(test_int);

    case test_int is
      --    testing text to number conversion :
      when  1 => report integer'image(Num_to_int("",         " mustfail 1|")); -- invalid number, empty string
      when  2 => report integer'image(Num_to_int("a",        " mustfail 2|")); -- invalid digit 'a'
      when  3 => report integer'image(Num_to_int("h",        " mustfail 3|")); -- invalid digit 'h'
      when  4 => report integer'image(Num_to_int("-",        " mustfail 4|")); -- invalid digit '-'
      when  5 => report integer'image(Num_to_int("111111111111111"," mustfail 5|")); -- invalid number, overflow
      when  6 => report integer'image(Num_to_int("FFFFFFFFH"," mustfail 6|")); -- invalid number, overflow
      when  7 => report integer'image(Num_to_int("333B",     " mustfail 7|")); -- wrong base '3'
      when  8 => report integer'image(Num_to_int("2468A",    " mustfail 8|")); -- wrong base 'A'
      when  9 => report integer'image(Num_to_int("B",        " mustfail 9|")); -- invalid number, no digit? 'B'
      when 10 => report integer'image(Num_to_int("-H",       " mustfail 10|")); -- invalid digit '-'

      -- trying to trigger errors in these functions:
      when 11 => report "XOR  R1  123     = " & SLV_to_bin(Assemble_RIO (Op_XOR,  Reg_R1,   123          ));
      when 12 => report "IN   R1  1234    = " & SLV_to_bin(Assemble_RIO (Op_IN,   Reg_R1,   1234         ));
      when 13 => report "IN   R1 -1234    = " & SLV_to_bin(Assemble_RIO (Op_IN,   Reg_R1,  -1234         ));
      when 14 => report "OUT  A1  123     = " & SLV_to_bin(Assemble_RI8 (Op_OUT,  Reg_A1,   123          ));
      when 15 => report "OR   A1  1234    = " & SLV_to_bin(Assemble_RI8 (Op_OR,   Reg_A1,   1234         ));
      when 16 => report "OR   A1 -1234    = " & SLV_to_bin(Assemble_RI8 (Op_OR,   Reg_A1,  -1234         ));
      when 17 => report "OUT  D1   3      = " & SLV_to_bin(Assemble_RI3 (Op_OUT,  Reg_D1,     3          ));
      when 18 => report "AND  D1   34     = " & SLV_to_bin(Assemble_RI3 (Op_AND,  Reg_D1,     34         ));
      when 19 => report "OUT  D1   3      = " & SLV_to_bin(Assemble_RI3C(Op_OUT,  Reg_D1,     3 , Cond_NC));
      when 20 => report "ANDN D1   34     = " & SLV_to_bin(Assemble_RI3C(Op_ANDN, Reg_D1,     34, Cond_NC));
      when 21 => report "OUT  A2  R2      = " & SLV_to_bin(Assemble_RR  (Op_OUT,  Reg_A2, Reg_R2         ));
      when 22 => report "IN   A2  R2 IFN2 = " & SLV_to_bin(Assemble_RRC (Op_IN,   Reg_A2, Reg_R2, Cond_N2));

      -- Line Assembler :
      when 23 => report SLV_to_bin(Assemble("IN R1 1234",  " mustfail 11|"));    -- Imm9 value too large
      when 24 => report SLV_to_bin(Assemble("IN R1 -1234", " mustfail 12|"));    -- Imm9 value too large
      when 25 => report SLV_to_bin(Assemble("IN R1 R1",    " mustfail 13|"));    -- invalid digit 'R'
      when 26 => report SLV_to_bin(Assemble("IN R1",       " mustfail 14|"));    -- missing argument ?
      when 27 => report SLV_to_bin(Assemble("IN RA 123",   " mustfail 15|"));    -- unknown register name

      -- older code, not very useful but good to keep:
      when 28 => result := Assemble("         AAA", " mustfail 16|"); -- "invalid instruction ?"
      when 29 => result := Assemble("AAAAAAAAAAAA", " mustfail 17|"); -- " ?"
      when 30 => result := Assemble("     ,  "    , " mustfail 18|");     -- "invalid character ?"
      when 31 => result := Assemble("     A  "    , " mustfail 19|");     -- "invalid opcode" (too short, at least 2 chars)
      when 32 => result := Assemble("AAAA AA AA A", " mustfail 20|"); -- "unknown opcode 'AAAA'"
      when 33 => result := Assemble(" MOV        ", " mustfail 21|"); -- "Missing destination register ?"
      when 34 => result := Assemble("MOV * R1 123", " mustfail 22|"); -- "invalid character"
      when 35 => result := Assemble("IN  R1; plop", " mustfail 23|"); -- "Mistyped destination ?"
      when 36 => result := Assemble("  IN  R  123", " mustfail 24|"); -- "invalid destination register"
      when 37 => result := Assemble(" IN ABC  123", " mustfail 25|"); -- "invalid destination register"
      when 38 => result := Assemble("IN  R1*  123", " mustfail 26|"); -- "invalid character"
      when 39 => result := Assemble("IN RQ    123", " mustfail 27|"); -- "unknown register"
      when 40 => result := Assemble("IN     R1   ", " mustfail 28|");  -- "Missing source operand"
      when 41 => result := Assemble("IN R1  123 *", " mustfail 29|");
      when 42 => result := Assemble("IN R1  123 R", " mustfail 30|");
      when 43 => result := Assemble("IN R1  123* ", " mustfail 31|");
      when 44 => result := Assemble("IN R1  123 *", " mustfail 32|");
      when 45 => result := Assemble("IN R1  123 R", " mustfail 33|");
      when 46 => result := Assemble("IN R1  123* ", " mustfail 34|");
      when 47 => result := Assemble("a b c d e f ", " mustfail 35|"); -- too many arguments
      when 48 => result := Assemble("a b c * ;;; ", " mustfail 36|"); -- invalid character '*'
      when 49 => result := Assemble("abcdefghijklmn", " mustfail 37|"); -- word too long
      when 50 => result := Assemble("comment ; plop", " mustfail 38|"); -- testing the comments

      when others => null;
    end case;

    report "Code " & test_str & " : IF YOU GET HERE THEN SOMETHING WRONG HAPPENED";
    -- but don't make it a failure, or it will appear as a success....
    wait;
  end process;

end fail;

