-- file YGREC8/mustpass.vhdl
-- created dim. nov. 19 08:15:10 CET 2017 by Yann Guidon / ygdes.com
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;
    use work.ygrec8_utils.all;
    use work.ygrec8_asm.all;

entity mustpass is
end mustpass;

architecture plip of mustpass is
begin

  bench: process
    variable j, k, errors: integer := 0;
    variable result, expected : SLV16;

    procedure pass_number(txt, ctx: string; val: integer) is
      variable e : integer;
    begin
      e := Num_to_int(txt, ctx);
      if e /= val then
        report ctx & "  " & txt & " : " & integer'image(e) & " /= " & integer'image(val);
        errors := errors+1;
      end if;
    end pass_number;

    procedure pass_assemble(ctx: string; val: SLV16; exp: SLV16) is
    begin
      if exp /= val then
        report ctx & " : " & SLV_to_bin(exp) & " /= " & SLV_to_bin(val);
        errors := errors+1;
      end if;
    end pass_assemble;

    procedure pass_instruction(txt, ctx: string; val: SLV16) is
      variable e : SLV16;
    begin
      e := Assemble(txt, ctx);
      if e /= val then
        report ctx & "  " & txt & " : " & SLV_to_bin(e) & " /= " & SLV_to_bin(val);
        errors := errors+1;
      end if;
    end pass_instruction;

  begin
    -- testing text to number conversion
    pass_number("AH",            " mustpass 1|", 10);
    pass_number("01101101H",     " mustpass 2|", 17830145);
    pass_number("01101101011011010110110101101101B"," mustpass 3|", 1835887981);
    pass_number("01101101",      " mustpass 4|", 1101101);
    pass_number("0",             " mustpass 5|", 0);
    pass_number("-01101101",     " mustpass 6|", -1101101);
    pass_number("7FFFFFFFH",     " mustpass 7|", 2147483647);
    pass_number("-7FFFFFFFH",    " mustpass 8|", -2147483647);

    pass_assemble("IN   R1 123      ", Assemble_RIO (Op_IN,   Reg_R1,    123         ), "1100100001111011");
    pass_assemble("OR   A1 123      ", Assemble_RI8 (Op_OR,   Reg_A1,    123         ), "0000001101111011");
    pass_assemble("AND  D1   3      ", Assemble_RI3 (Op_AND,  Reg_D1,      3         ), "0010000000001011");
    pass_assemble("ANDN D1   3      ", Assemble_RI3C(Op_ANDN, Reg_D1,      3, Cond_NC), "0011000010011011");
    pass_assemble("XOR  A2  R2      ", Assemble_RR  (Op_XOR,  Reg_A2, Reg_R2         ), "0001011000000101");
    pass_assemble("ADD  A2  R2 IFN2 ", Assemble_RRC (Op_ADD,  Reg_A2, Reg_R2, Cond_N2), "0111011011100101");

    pass_instruction("", " mustpass 1|", "UUUUUUUUUUUUUUUU");
    pass_instruction(" ; plop ! ", " mustpass 2|", "UUUUUUUUUUUUUUUU");
    pass_instruction("CALL R1 R2", " mustpass 3|", "1110100000000101");
    pass_instruction("IN R1 123", " mustpass 4|", "1100100001111011");
    pass_instruction("INV"   , " mustpass 11|", "1111111111111111");
    pass_instruction("NOP ; comment! ", " mustpass 12|", "0000000000000000");
    pass_instruction("or   D1 D1", " mustpass a0|", "0000000000000000");
    pass_instruction("xor  D1 D1", " mustpass a1|", "0001000000000000");
    pass_instruction("and  D1 D1", " mustpass a2|", "0010000000000000");
    pass_instruction("andn D1 D1", " mustpass a3|", "0011000000000000");
    pass_instruction("cmpu D1 D1", " mustpass a4|", "0100000000000000");
    pass_instruction("cmps D1 D1", " mustpass a5|", "0101000000000000");
    pass_instruction("sub  D1 D1", " mustpass a6|", "0110000000000000");
    pass_instruction("add  D1 D1", " mustpass a7|", "0111000000000000");
    pass_instruction("shr  D1 D1", " mustpass a8|", "1000000000000000");
    pass_instruction("shl  D1 D1", " mustpass a9|", "1001000000000000");
    pass_instruction("sar  D1 D1", " mustpass a10|", "1010000000000000");
    pass_instruction("rol  D1 D1", " mustpass a11|", "1011000000000000");
    pass_instruction("in   D1  0", " mustpass a12|", "1100000000000000");
    pass_instruction("out  D1  0", " mustpass a13|", "1101000000000000");
    pass_instruction("call D1 D1", " mustpass a14|", "1110000000000000");
    pass_instruction("mov  D1 D1", " mustpass a15|", "1111000000000000");
    pass_instruction("OR D1 A1",      " mustpass a17|", "0000000000000001");
    pass_instruction("OR D2 A2",      " mustpass a18|", "0000010000000011");
    pass_instruction("OR R1 R2",      " mustpass a19|", "0000100000000101");
    pass_instruction("OR R3 PC",      " mustpass a20|", "0000110000000111");
    pass_instruction("OR D1 D1 ALWS", " mustpass a21|", "0000000000000000");
    pass_instruction("OR D1 D1 IFNC", " mustpass a22|", "0000000000010000");
    pass_instruction("OR D1 D1 IFN" , " mustpass a23|", "0000000000100000");
    pass_instruction("OR D1 D1 IFNZ", " mustpass a24|", "0000000000110000");
    pass_instruction("OR D1 D1 IF0" , " mustpass a25|", "0000000001000000");
    pass_instruction("OR D1 D1 IF1" , " mustpass a26|", "0000000001010000");
    pass_instruction("OR D1 D1 IF2" , " mustpass a27|", "0000000001100000");
    pass_instruction("OR D1 D1 IF3" , " mustpass a28|", "0000000001110000");
    pass_instruction("OR D1 D1 NEVR", " mustpass a29|", "0000000010000000");
    pass_instruction("OR D1 D1 IFC" , " mustpass a30|", "0000000010010000");
    pass_instruction("OR D1 D1 IFP" , " mustpass a31|", "0000000010100000");
    pass_instruction("OR D1 D1 IFZ" , " mustpass a32|", "0000000010110000");
    pass_instruction("OR D1 D1 IFN0", " mustpass a33|", "0000000011000000");
    pass_instruction("OR D1 D1 IFN1", " mustpass a34|", "0000000011010000");
    pass_instruction("OR D1 D1 IFN2", " mustpass a35|", "0000000011100000");
    pass_instruction("OR D1 D1 IFN3", " mustpass a36|", "0000000011110000");

    assert errors = 0
      report integer'image(errors) & " UNEXPECTED RESULT(S)"
      severity failure;
    wait;
  end process;

end plip;
