-- YGREC8/tb1.vhdl
-- created mer. nov. 15 05:44:44 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version jeu. nov. 16 02:21:35 CET 2017 : uses SLV shorthands from YGREC8/YGREC8_def.vhdl
-- Released under the GNU AGPLv3 license
--
--
-- A first crude test of the opcodes and their behaviour.
-- It's just feeding fixed codes manually...

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;
    use work.ygrec8_utils.all;
    use work.ygrec8_asm.all;

entity Core_tb is
end Core_tb;

architecture tb of Core_tb is
  -- testbench interface signals:
  signal clk, reset, enable_cycle, CB0, CB1, CB2, CB3,
    nC, C, n_S, S, nZ, Z, write_enable : SL;
  signal InstructionWord : SLV16;
  signal IN_PORT, PC, NextPC, WriteBack, DST, SRC : SLV8;
begin

  tb: entity YGREC8_Core port map (
    clk             =>             clk,
    reset           =>           reset,
    enable_cycle    =>    enable_cycle,
    CB0             =>             CB0,
    CB1             =>             CB1,
    CB2             =>             CB2,
    CB3             =>             CB3,
    InstructionWord => InstructionWord,
    IN_PORT         =>         IN_PORT,
    PC              =>              PC,
    NextPC          =>          NextPC,
    WriteBack       =>       WriteBack,
    DST             =>             DST,
    SRC             =>             SRC,
    nC              =>              nC,
    C               =>               C,
    nS              =>             n_S,
    S               =>               S,
    nZ              =>              nZ,
    Z               =>               Z,
    write_enable    =>    write_enable
  );

  bench: process
    variable j, k: integer := 0;
    variable expected : std_logic_vector(7 downto 0);

    procedure ShowState is
    begin
      report  "/R=" & std_logic'image(reset)(2)
           & " EN=" & std_logic'image(enable_cycle)(2)
           & " WE=" & std_logic'image(write_enable)(2)

           & " Inst=" & SLV_to_bin(InstructionWord)
           & " SRC=" & SLV_to_bin(SRC)
           & " DST=" & SLV_to_bin(DST)
           & " WB=" & SLV_to_bin(WriteBack)
           & " nPC=" & SLV_to_bin(NextPC)
           & " PC=" & SLV_to_bin(PC)

        --   & " IN=" & SLV_to_bin(IN_PORT)
        --   & " @=" & SLV_to_bin(ADR_PORT)

           &"  nC=" & std_logic'image(nC)(2)
           &  " C=" & std_logic'image(C)(2)
           & " nS=" & std_logic'image(n_S)(2)
           &  " S=" & std_logic'image(S)(2)
           & " nZ=" & std_logic'image(nZ)(2)
           &  " Z=" & std_logic'image(Z)(2)
        --   & " CB=" & std_logic'image(CB0)(2)
        --   &          std_logic'image(CB1)(2)
        --   &          std_logic'image(CB2)(2)
        --   &          std_logic'image(CB3)(2)
        ;
    end ShowState;

--      if Op="IN"  then INST(15 downto 12) := Op_IN;  else
--      if Op="OUT" then INST(15 downto 12) := Op_OUT; else
--        assert false report "Assemble_IO : unknown opcode " & Op severity failure;
--    function Assemble_I3 (Op: SLV4; dst port_nr: integer) return SLV16 is
--      variable INST: SLV16;
--      variable nr : integer := port_nr;
--    begin
--      INST(15 downto 12) := Op;
--      assert (nr < 512) and (nr >= -256)
--          report "Assemble_IO : port number is out of range :" & integer'image(nr)
--           severity failure;
--      end if; end if;

  begin
    -- reset
    clk <= '0';
    reset <= '0';
    wait for 1 ns;
    ShowState;
    reset <= '1';
    wait for 1 ns;
    ShowState;
    enable_cycle <= '1';
    wait for 1 ns;
    ShowState;

    -- MOV R1 6Ch (imm8)
    InstructionWord <= Op_MOV & Reg_R1 & '1' & "01101101";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01101101" report "unexpected result" severity failure;

    -- MOV R2 2 (Imm3, always)
    InstructionWord <= Op_MOV & Reg_R2 & '0' & Cond_A & '1' & "010";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="00000010" report "unexpected result" severity failure;

    -- ADD R2 R1 (always)
    InstructionWord <= Op_ADD & Reg_R2 & '0' & Cond_A & '0' & Reg_R1;
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01101111" report "unexpected result" severity failure;

    -- ADD R2 1 (always)
    InstructionWord <= Op_ADD & Reg_R2 & '0' & Cond_A & '1' & "001";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01110000" report "unexpected result" severity failure;

    -- MOV A1 R2  (always)
    InstructionWord <= Op_MOV & Reg_A1 & '0' & Cond_A & '0' & Reg_R2;
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01110000" report "unexpected result" severity failure;

    -- MOV D1 42h (imm8)
    InstructionWord <= Op_MOV & Reg_D1 & '1' & "01000010";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01000010" report "unexpected result" severity failure;

    -- ADD A1 1 (imm3 always)
    InstructionWord <= Op_ADD & Reg_A1 & '0' & Cond_A & '1' & "001";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01110001" report "unexpected result" severity failure;

    -- MOV D1 43h (imm8)
    InstructionWord <= Op_MOV & Reg_D1 & '1' & "01000011";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01000011" report "unexpected result" severity failure;

    -- ADD A1 -1 (imm3 always)
    InstructionWord <= Op_ADD & Reg_A1 & '0' & Cond_A & '1' & "111";
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    assert DST="01110000" report "unexpected result" severity failure;

    -- MOV R3 D1 (always)
    InstructionWord <= Op_MOV & Reg_R3 & '0' & Cond_A & '0' &  Reg_D1;
    wait for 1 ns;    ShowState;
    clk <= '1';    wait for 1 ns;
    clk <= '0';    wait for 1 ns; ShowState;
    wait;
  end process;

end tb;

