-- YGREC8/DP8/DP8.vhdl
-- created mar. oct. 23 04:13:44 CEST 2018
-- merged register set (R7) and some datapath

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;

entity DP8 is
  port(
    Clk, Reset,
    D1, D2,                                                   -- data coming from the data memory block
    WrEnA1, WrEnA2, WrEnR1, WrEnR2, WrEnR3, WrEnPC,   -- write enable lines for the DFF
    SndS0, SndS1, SndS2, SriS0, SriS1, SriS2,                -- Read Select for the 2 ports
    AluIn, PortIn, Imm8, Inc8In,                               -- external data inputs from other datapath units
    OpcCall, OpcIn, RImm,                                     -- control signals from the instruction word
    WrA1, WrA2,                                               -- select between A0/A1 and Result
      : in SL;

    Addr1, Addr2,  // go to the data memory block
    PCout, NextPC,    // Go to INC8 and program memory block
    SRI, SND       // Register set read outputs
     : out SL);
end DP8;

architecture classic of DP8 is
  signal A1,   A2,   R1,   R2,   R3,   PC,    ;  -- The registers
         Result, SelPC, Rin, PCin   -- The registers' inputs
           : SL;

begin
  process(Reset, Clk)
  begin
    if Reset='0' then
      A1 <= '0';
      A2 <= '0';
      R1 <= '0';
      R2 <= '0';
      R3 <= '0';
      PC <= '1';  -- Reset to -1 so nextPC=0 on reset
    else
      if rising_edge(Clk) then
        if WrEnA1='1' then  A1<=A1in; end if;
        if WrEnA2='1' then  A2<=A2in; end if;
        if WrEnR1='1' then  R1<=R1in; end if;
        if WrEnR2='1' then  R2<=R2in; end if;
        if WrEnR3='1' then  R3<=R3in; end if;
        if WrEnPC='1' then  PC<=PCin; end if;
      end if;  -- Clk
    end if;
  end process;

  -- update the write-only outputs
  PCout  <= PC;
  NextPC <= PCin;


  Result <= AluIn when OpcIn='0' else PortIn;
  SelPC  <= 
 <= 



    <= R1 when SriS0='0' else R2 ;     <= R1 when SndS0='0' else R2;
    <= R3 when SriS0='0' else PC ;     <=  when SndS0='0' else  ;
    <= D1 when SriS1='0' else D2 ;     <= D1 when SndS1='0' else D2;
    <=   when SriS1='0' else     ;     <=  when SndS1='0' else  ;

    <=   when SriS0='0' else     ;     <=  when SndS0='0' else  ;
    <=   when SriS1='0' else     ;     <=  when SndS1='0' else  ;

  SRI <=   when SriS2='0' else     ;    SND <=  when SndS2='0' else  ;

end classic;
