-- YGREC8/SHL8.vhdl
-- created ven. nov. 10 17:35:52 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version ven. nov. 10 18:21:38 CET 2017 (so wrote v1 < 1h)
-- version jeu. nov. 16 03:09:25 CET 2017 : uses SLV shorthands from YGREC8/YGREC8_def.vhdl
-- Released under the GNU AGPLv3 license
--
-- This is the optional SHuffLing unit of YGREC8,
-- a barrel shifter/rotater operating on 8 bits.
--
-- OPCODES:
-- F3 F2 F1 F0       tested
-- 1  0  0  0   SHR     *
-- 1  0  0  1   SHL     *
-- 1  0  1  0   SAR     *
-- 1  0  1  1   ROL     *

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.ygrec8_def.all;

entity SHL8 is
  port( F0, F1 : in SL;
    SRC	: in SLV3;
    DST : in SLV8;
    SHL : out SLV8);
end SHL8;

architecture rtl of SHL8 is
  Signal L0, L1 : SLV8;
  Signal MSB, LSB : SLV4;

begin
  -- Prepare the control signals:
  MSB <= (others => DST(7) and F1 and not F0); -- SAR mask
  LSB <= (others =>            F1 and     F0); -- ROL mask

  L0 <= DST when SRC(0)='0'                    -- Pass
   else MSB(0) & DST(7 downto 1) when F0='0'   -- SHR/SAR
   else DST(6 downto 0) & (DST(7) AND LSB(0)); -- SHL/ROL

  L1 <= L0 when SRC(1)='0'                                     -- Pass
   else MSB(1 downto 0) & L0(7 downto 2) when F0='0'           -- SHR/SAR
   else L0(5 downto 0) & (L0(7 downto 6) AND LSB(1 downto 0)); -- SHL/ROL

  SHL<= L1 when SRC(2)='0'                                     -- Pass
   else MSB(3 downto 0) & L1(7 downto 4) when F0='0'           -- SHR/SAR
   else L1(3 downto 0) & (L1(7 downto 4) AND LSB(3 downto 0)); -- SHL/ROL
end rtl;
