-- proasic3/PA3_components.vhdl
-- version 20181017: more gates !
--
-- This is just a little file that implements some gates and stuff from the
-- standard ProASIC3 cell library (Actel/Microsemi) in order
-- to simulate with a standalone tool (ghdl)
--
-- It's used to check equivalence with files that start with these lines :
-- library proasic3;
--     use proasic3.all;
-- Since the original is made of proprietary code, I'm just rewriting the gates that I use,
-- along with a simple RTL implementation suitable for cycle-acurate simulations.

Library ieee;
    use ieee.std_logic_1164.all;

package proasic3 is

  component AND2     port(A, B    : in std_logic; Y : out std_logic); end component;
  component AND2A    port(A, B    : in std_logic; Y : out std_logic); end component;
  component AND3     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AND3A    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AND3B    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AND3C    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AO1      port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AO1A     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AOI1D    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AX1      port(A, B, C : in std_logic; Y : out std_logic); end component;
  component AX1A     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component INV      port(A       : in std_logic; Y : out std_logic); end component;
  component MX2      port(A, B, S : in std_logic; Y : out std_logic); end component;
  component MX2A     port(A, B, S : in std_logic; Y : out std_logic); end component;
  component MX2C     port(A, B, S : in std_logic; Y : out std_logic); end component;
  component NAND2    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NAND3    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NAND3A   port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NAND3B   port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NOR2     port(A, B    : in std_logic; Y : out std_logic); end component;
  component NOR2A    port(A, B    : in std_logic; Y : out std_logic); end component;
  component NOR3     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NOR3A    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component NOR3B    port(A, B, C : in std_logic; Y : out std_logic); end component;
  component OA1A     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component OA1C     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component OR2      port(A, B    : in std_logic; Y : out std_logic); end component;
  component OR3      port(A, B, C : in std_logic; Y : out std_logic); end component;
  component OR3A     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component OR3B     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component XA1      port(A, B, C : in std_logic; Y : out std_logic); end component;
  component XA1A     port(A, B, C : in std_logic; Y : out std_logic); end component;
  component XNOR2    port(A, B    : in std_logic; Y : out std_logic); end component;
  component XO1      port(A, B, C : in std_logic; Y : out std_logic); end component;
  component XOR2     port(A, B    : in std_logic; Y : out std_logic); end component;
  component XOR3     port(A, B, C : in std_logic; Y : out std_logic); end component;

  component CLKBUFF  port(                        Y : out std_logic); end component;
--  alias   CLKBUF_LVCMOS33 is CLKBUFF;
  component CLKINT   port(A       : in std_logic; Y : out std_logic); end component;

  component DFN1     port(D,    CLK      : in std_logic; Q : out std_logic); end component;
  component DFN1E1   port(D, E, CLK      : in std_logic; Q : out std_logic); end component;
  component DFN1E1C0 port(D, E, CLK, CLR : in std_logic; Q : out std_logic); end component;
  component DFN1E1C1 port(D, E, CLK, CLR : in std_logic; Q : out std_logic); end component;

  component RAM4K9 port(
    ADDRA11, ADDRA10, ADDRA9, ADDRA8, ADDRA7, ADDRA6,
    ADDRA5,  ADDRA4,  ADDRA3, ADDRA2, ADDRA1, ADDRA0,
    ADDRB11, ADDRB10, ADDRB9, ADDRB8, ADDRB7, ADDRB6,
    ADDRB5,  ADDRB4,  ADDRB3, ADDRB2, ADDRB1, ADDRB0,
    DINA8, DINA7, DINA6, DINA5, DINA4, DINA3, DINA2, DINA1, DINA0,
    DINB8, DINB7, DINB6, DINB5, DINB4, DINB3, DINB2, DINB1, DINB0,
    WIDTHA0, WIDTHA1, WIDTHB0, WIDTHB1, PIPEA, PIPEB, WMODEA, WMODEB,
    BLKA, BLKB, WENA, WENB, CLKA, CLKB, RESET : in std_logic := 'U';
    DOUTA8, DOUTA7, DOUTA6, DOUTA5, DOUTA4, DOUTA3, DOUTA2, DOUTA1, DOUTA0,
    DOUTB8, DOUTB7, DOUTB6, DOUTB5, DOUTB4, DOUTB3, DOUTB2, DOUTB1, DOUTB0 : out std_logic);
  end component;

  -- the FIFO is not included here, there is a simple wrapper though in the directory.

end proasic3;
